-- Guest Access.applescript
-- Guest Access

--  Created by Nico Rohrbach on 18.06.06.
--  Copyright 2006 Superpixel. All rights reserved.

global image_alert
global image_locked
global image_status
global image_unlocked

global guestaccess_state

-- property action_button_state : "disable" -- disable, enable
property alternate_message : false

property unverified_username : ""
property unverified_passwd : ""

property the_username : ""
property the_passwd : ""


(* ==== Event Handlers ==== *)


on will finish launching theObject
	set image_alert to load image "caution-16"
	set image_locked to load image "status-locked"
	set image_status to load image "status"
	set image_unlocked to load image "status-unlocked"
end will finish launching


on launched theObject
	-- prepare the auth panel
	set contents of text field "name" of window "auth" to ""
	set contents of text field "password" of window "auth" to ""
	set first responder of window "auth" to text field "name" of window "auth"
	set uses threaded animation of progress indicator "spinner" of window "main" to true
	-- prefill if we can
	if currentUserIsAdmin() is true then
		set contents of text field "name" of window "auth" to currentUsersLongName()
		set first responder of window "auth" to text field "password" of window "auth"
	end if
	-- reset the status image
	delete image of image view "authminiicon" of window "auth"
	-- show the panel
	center window "auth"
	show window "auth"
end launched


on clicked theObject
	
	if name of theObject is "quit" then
		quit me
		
	else if name of theObject is "ok" then
		set unverified_username to contents of text field "name" of window "auth"
		set unverified_passwd to contents of text field "password" of window "auth"
		
		set returnedPasswd to verifyPassword(unverified_username, unverified_passwd)
		
		if returnedPasswd is false then
			beep
			set image of image view "authminiicon" of window "auth" to image_alert
			my authStatus("Identification failed. Try again.")
			-- delete and possibly refill the fields
			set contents of text field "name" of window "auth" to ""
			set contents of text field "password" of window "auth" to ""
			set first responder of window "auth" to text field "name" of window "auth"
			-- prefill if we can
			if currentUserIsAdmin() is true then
				set contents of text field "name" of window "auth" to currentUsersLongName()
				set first responder of window "auth" to text field "password" of window "auth"
			end if
			
		else
			-- populate variables with verified username and password
			set the_username to unverified_username
			set the_passwd to returnedPasswd
			-- close the auth panel and show main window
			center window "main"
			show window "main"
			close panel window "auth"
			my checkService()
		end if
		
	else if name of theObject is "opensharing" then
		my showPrefPanel()
		
	else if name of theObject is "action" then
		if title of theObject is "Enable" then
			my setGuestAccess("enable") -- status {"enable", "disable"}
			my checkService()
			
		else if title of theObject is "Disable" then
			my setGuestAccess("disable") -- status {"enable", "disable"}
			my checkService()
			
		end if
		
	end if
end clicked


on should quit after last window closed theObject
	return true
end should quit after last window closed


on choose menu item theObject
	if name of theObject is "refresh" then
		my checkService()
	end if
end choose menu item



(* ==== Private Handlers ==== *)


on checkService()
	set enabled of button "action" of window "main" to false
	tell progress indicator "spinner" of window "main" to start
	my mainStatus("Get current Status ", "")
	set image of image view "status" of window "main" to image_status
	-- wait a second
	delay 0.5
	-- do the job
	try
		set guestaccess_state to 
			(do shell script "/usr/bin/defaults read /Library/Preferences/com.apple.AppleFileServer guestAccess" user name 
				the_username password the_passwd with administrator privileges) as number
	on error err_msg
		beep
		log (err_msg as text)
		tell progress indicator "spinner" of window "main" to stop
		my mainStatus("Can't get current Status!", "An error occured. Try again later or see error log for details.")
		return
	end try
	
	tell progress indicator "spinner" of window "main" to stop
	
	-- guest access is enabled
	if guestaccess_state is 1 then
		set image of image view "status" of window "main" to image_unlocked
		if alternate_message is false then
			my mainStatus("Guest access to this Mac is enabled.", 
				"Everyone has access to your Mac over the network. This can be a potential security hole. If you don't need guest access, you can disable it.")
		else if alternate_message is true then
			my mainStatus("Guest access to this Mac is enabled.", 
				"You must restart \"Personal File Sharing\" to apply this changes! You can do this in the Sharing Panel of System Preferences by stop and start the service.")
			-- set text color of text field "subinfo" of window "main" to {45633, 0, 0}
		end if
		tell window "main"
			set title of button "action" to "Disable"
		end tell
		
		-- guest access is disabled
	else if guestaccess_state is 0 then
		set image of image view "status" of window "main" to image_locked
		if alternate_message is false then
			my mainStatus("Guest access to this Mac is disabled.", 
				"No guest has access to your Mac over the network. If you need guest access, you can enable it.")
		else if alternate_message is true then
			my mainStatus("Guest access to this Mac is disabled.", 
				"You must restart \"Personal File Sharing\" to apply this changes! You can do this in the Sharing Panel of System Preferences by stop and start the service.")
			-- set text color of text field "subinfo" of window "main" to {45633, 0, 0}
		end if
		tell window "main"
			set title of button "action" to "Enable"
		end tell
	end if
	
	tell window "main"
		set enabled of button "action" to true
		if alternate_message is false then
			set visible of button "opensharing" to false
		else if alternate_message is true then
			set visible of button "opensharing" to true
		end if
	end tell
	
end checkService


(*
on afpActive()
	tell application "System Events" to set proc_list to name of every process whose (visible is false)
	if proc_list contains "AppleFileServer" then
		--
	end if
end afpActive
*)


on setGuestAccess(the_status) -- status {"enable", "disable"}
	if the_status is "enable" then
		try
			do shell script 
				"/usr/bin/defaults write /Library/Preferences/com.apple.AppleFileServer guestAccess -bool true" user name 
				the_username password the_passwd with administrator privileges
		on error err_msg
			log (err_msg as text)
			return 0
		end try
		set alternate_message to true
		return 1
	else if the_status is "disable" then
		try
			do shell script 
				"/usr/bin/defaults write /Library/Preferences/com.apple.AppleFileServer guestAccess -bool false" user name 
				the_username password the_passwd with administrator privileges
		on error err_msg
			log (err_msg as text)
			return 0
		end try
		set alternate_message to true
		return 1
	end if
end setGuestAccess


on mainStatus(first_text, second_text)
	set contents of text field "maininfo" of window "main" to first_text
	set contents of text field "subinfo" of window "main" to second_text
end mainStatus


on authStatus(the_message)
	set contents of text field "authstatus" of window "auth" to the_message
end authStatus


on showPrefPanel()
	tell application "System Preferences"
		activate
		reveal anchor "Services_PersonalFileSharing" of pane id "com.apple.preferences.sharing"
	end tell
end showPrefPanel


on currentUsersLongName()
	try
		tell application "System Events" to set temp to current user's full name
		return temp
	on error
		log "can't get current users long name"
		return ""
	end try
end currentUsersLongName


on currentUserIsAdmin()
	set shortUsername to do shell script "/usr/bin/whoami"
	set temp to do shell script "/usr/bin/id" & space & shortUsername
	if temp contains "admin" then
		return true
	else
		return false
	end if
end currentUserIsAdmin


on verifyPassword(usrname, passwd)
	try
		do shell script "sudo -k"
		do shell script "sudo -v" user name usrname password passwd with administrator privileges
		return passwd
	on error errMsg number errNum
		if errMsg contains ("Sorry, try again" as Unicode text) then
			-- display dialog "The password you entered is incorrect" buttons {"OK"} default button 1
			return false
		else if errNum is -927 then
			-- handle the blank password. You can return any string, sudo will accept everything.
			-- Its AppleScript that won't accept the empty string
			try
				set arbitraryPassword to "blank"
				do shell script "sudo -v" user name 
					usrname password arbitraryPassword with administrator privileges
				return arbitraryPassword
			on error msg
				--display dialog "The password you entered is incorrect" buttons {"OK"} default button 1
				return false
			end try
		else
			log errMsg & return & errNum
			return false
		end if
	end try
end verifyPassword
